@@ -29,6 +29,10 @@
 #'@param lower.tail  Logical value indicating if results are \eqn{P[X \leq x]}
 #'                   (if \code{TRUE}; default) or \eqn{P[X > x]} (if 
 #'                   \code{FALSE}).
+#'@param generator   Character string that specifies the random number
+#'                   generator and must either be \code{"Sample"} (default) or
+#'                   \code{"Bernoulli"} (abbreviations are allowed). See
+#'                   Details for more information.
 #'
 #'@details
 #'See the references for computational details. The \emph{Divide and Conquer}
@@ -52,6 +56,18 @@
 #'In some special cases regarding the values of \code{probs}, the \code{method}
 #'parameter is ignored (see Introduction vignette).
 #'
+#'Random numbers can be generated in two ways. The \code{"Sample"} method
+#'uses \code{R}'s \code{sample} function to draw random values according to
+#'their probabilities that are calculated by \code{dgpbinom}. The
+#'\code{"Bernoulli"} procedure ignores the \code{method} parameter and
+#'simulates Bernoulli-distributed random numbers according to the probabilities
+#'in \code{probs} and sums them up. It is a bit slower than the \code{"Sample"}
+#'generator, but may yield better results, as it allows to obtain observations
+#'that cannot be generated by the \code{"Sample"} procedure, because
+#'\code{dgpbinom} may compute 0-probabilities, due to rounding, if the length
+#'of \code{probs} is large and/or its values contain a lot of very small
+#'values.
+#'
 #'@return
 #'\code{dpbinom} gives the density, \code{ppbinom} computes the distribution
 #'function, \code{qpbinom} gives the quantile function and \code{rpbinom}
@@ -375,20 +391,18 @@ qpbinom <- function(p, probs, wts = NULL, method = "DivideFFT", lower.tail = TRU
   # order 'p' depending on if they are lower tail probabilities; save order
   ord <- order(p, decreasing = !lower.tail)
   p.s <- p[ord]
-  # convert 'lower.tail' to a numeric variable
-  lower.tail <- as.numeric(lower.tail)
   # negative sign, if lower.tail = TRUE
   sign <- (-1)^(1 - lower.tail)
   cdf <- sign*cdf
   # compute quantiles
   for(i in 1:length(p.s)){
     # handle 0's and 1's
-    if(p.s[i] == 1 - lower.tail){
-      res[i] <- (1 - lower.tail) * max(0, sum(probs == 1) - 1)
+    if(p.s[i] == as.numeric(!lower.tail)){
+      res[i] <- ifelse(lower.tail, 0, max(0, sum(probs == 1) - 1))
       next
     }
-    if(p.s[i] == lower.tail){
-      res[i] <- size - lower.tail * length(which(probs == 0))
+    if(p.s[i] == as.numeric(lower.tail)){
+      res[i] <- size - ifelse(lower.tail, sum(probs == 0), 0)
       next
     }
     # find the cdf value smaller than or equal to current 'p.s' value
@@ -404,20 +418,46 @@ qpbinom <- function(p, probs, wts = NULL, method = "DivideFFT", lower.tail = TRU
 }
 
 #'@rdname PoissonBinomial-Distribution
-#'@importFrom stats runif
+#'@importFrom stats runif rbinom
 #'@export
-rpbinom <- function(n, probs, wts = NULL, method = "DivideFFT"){
+rpbinom <- function(n, probs, wts = NULL, method = "DivideFFT", generator = "Sample"){
   len <- length(n)
   if(len > 1) n <- len
   
   # check if 'n' is NULL
   if(is.null(n)) stop("'n' must not be NULL!")
   
-  # generate random probabilities
-  p <- runif(n)
+  # number of probabilities
+  len <- length(probs)
   
-  # compute quantiles (does checking for the other variables)
-  res <- qpbinom(p, probs, wts, method)
+  # check if 'probs' contains only probabilities
+  if(is.null(probs) || any(is.na(probs) | probs < 0 | probs > 1))
+    stop("'probs' must contain real numbers between 0 and 1!")
+  
+  # check if 'wts' contains only integers (zeros are allowed)
+  if(!is.null(wts) && any(is.na(wts) | wts < 0 | abs(wts - round(wts)) > 1e-07))
+    stop("'wts' must contain non-negative integers!")
+  
+  if(!is.null(wts) && length(wts) != len)
+    stop("'probs' and 'wts' (if not NULL) must have the same length!")
+  
+  ## expand 'probs' according to the counts in 'wts'
+  # if 'wts' is NULL, set it to be a vector of ones
+  if(is.null(wts))
+    wts <- rep(1, len)
+  
+  # expand 'probs'
+  probs <- rep(probs, wts)
+  
+  # make sure that the value of 'method' matches one of the implemented procedures
+  method <- match.arg(method, c("DivideFFT", "Convolve", "Characteristic", "Recursive", "Mean", "GeoMean", "GeoMeanCounter", "Poisson", "Normal", "RefinedNormal"))
+  
+  # make sure that the value of 'generator' matches one of the implemented procedures
+  generator <- match.arg(generator, c("Sample", "Bernoulli"))
+  
+  # generate random numbers
+  res <- switch(generator, Sample    = sample(0:length(probs), n, TRUE, dpbinom(NULL, probs, NULL, method)),
+                           Bernoulli = rpb_bernoulli(n, probs))
   
   # return results
   return(res)
